# -*- coding: iso-8859-15 -*-
#    This program compares 2 Zipf's law cumulated distributions
#    To generate the figure 3.3 of Sharing, it must be run as:
#    python figure3-3.py 0.5 1.4 5000
#    where 0.5 and 1.4 are the Zipf's laws parameters and 5000
#    is the number of works considered.
#
#    Copyright (C) 2006-2008-2011 - Philippe Aigrain
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.

from pylab import *

def power(x,a):
  return(x**(-a))

def main(argv):
  if len(argv) < 4:
    print ("Usage: %s <2 values of Zipf's law parameter, number of works>") % argv[0]
    print "Plots cumulated Zipf's law distributions for parameters and number of works given in argument"
    sys.exit(0)
# Initializing stuff
  a1 = float(argv[1])
  a2 = float(argv[2])
  N = int(argv[3])
  ind = zeros(N,float)
  for i in range(N):
    ind[i]=float(i)/(N/100)
  s1 = zeros(N,float)
  s2 = zeros(N,float)
  t1 = zeros(N,float)
  t2 = zeros(N,float)
# Computing laws without normalization
  for i in range(N):
      s1[i] = power(i+1,a1)
      s2[i] = power(i+1,a2)
# Normalization
  tot1 = s1[0]
  tot2 = s2[0]
  for i in range(1,N,1):
    tot1 += s1[i]
    tot2 += s2[i]
  for i in range(N):
    s1[i] = s1[i] / tot1
    s2[i] = s2[i] / tot2
  print tot1,tot2
# Normalized cumulated
  for i in range(N):
      if i > 0:
        t1[i] = s1[i]+t1[i-1]
        t2[i] = s2[i]+t2[i-1]
      else:
        t1[i] = s1[i]
        t2[i] = s2[i]
  plotpar = {'axes.labelsize': 24,
               'text.fontsize': 24,
               'legend.fontsize': 20,
               'xtick.labelsize': 22,
               'ytick.labelsize': 22,
               'text.usetex': False}
  rcParams.update(plotpar)
  close(1)
  figure(num=1, figsize=(16,12), facecolor='w', edgecolor='k')
# Plotting
# Replace by following line for BW figure
  (l1,l2) = plot(ind,t1,'k-',ind,t2,'k--',lw=2,antialiased=True)
# Replace by next line for a color figure
  (l1,l2) = plot(ind,t1,'b-',ind,t2,'g--',lw=2,antialiased=True)
  annotate('93% of access goes to 5% most popular works', (6.0,0.9), xytext=None, xycoords='data', textcoords='data', arrowprops=None, fontsize=20)
  annotate('22% of access goes to 5% most popular works', (6.0,0.18), xytext=None, xycoords='data', textcoords='data', arrowprops=None, fontsize=20)
  axvline(x=5.0, ymin=0.0, ymax=1.0, color='r',linestyle='dotted', lw=2)
  xlabel('decreasing popularity ranks (percentage)')
  yaxislabel = "Cumulated Zipf's laws for %d works" % (N)
  ylabel(yaxislabel)
  ylim(0.0, 1.0)
  ylabel(yaxislabel)
  legend( (l1,l2),  ("parameter: %3.1f" % a1,"parameter: %3.1f" % a2), loc='lower right')
  grid(True)
  savefig('illustration.png')
  show()

if __name__ == '__main__':
    main(sys.argv)
