# -*- coding: iso-8859-15 -*-
#    This program compares 4 Zipf's law cumulative distributions.
#    The 4 laws have a common universe. To generate figure 7.2
#    To generate figure 7.2 of "Sharing", one has to run:
#    python figure7-2.py 1.1 1.0 0.9 0.8 2000000
#
#    Copyright (C) 2006-2008 - Philippe Aigrain
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.

from pylab import *

def power(x,a):
  return(x**(-a))
  
def harmonic(s,n):
  k=0.0;
  for i in arange (1.0,n+1.0,1.0):
    k += pow(i,-s);
  return k;  

def main(argv):
  if len(argv) < 6:
    print ("Usage: %s <4 values of Zipf's law parameter, number of works>") % argv[0]
    print "Plots cumulated Zipf's law distributions CDF for parameters and number of works given in argument"
    sys.exit(0)
# Initializing stuff
  a1 = float(argv[1])
  a2 = float(argv[2])
  a3 = float(argv[3])
  a4 = float(argv[4])
  N = int(argv[5])
  ind = range(1,N+1,1)
  s1 = zeros(N,float)
  s2 = zeros(N,float)
  s3 = zeros(N,float)
  s4 = zeros(N,float)
  t1 = zeros(N,float)
  t2 = zeros(N,float)
  t3 = zeros(N,float)
  t4 = zeros(N,float)
# Computing laws without normalization
  for i in range(N):
      s1[i] = (i+1)**(-a1)
      s2[i] = (i+1)**(-a2)
      s3[i] = (i+1)**(-a3)
      s4[i] = (i+1)**(-a4)
# Normalization
  ha1 = harmonic(a1,N)
  ha2 = harmonic(a2,N)
  ha3 = harmonic(a3,N)
  ha4 = harmonic(a4,N)
  for i in range(N):
    s1[i] = s1[i] / ha1
    s2[i] = s2[i] / ha2
    s3[i] = s3[i] / ha3
    s4[i] = s4[i] / ha4
# Normalized cumulated for the 4 laws
  for i in range(N):
      if i > 0:
        t1[i] = s1[i]+t1[i-1]
        t2[i] = s2[i]+t2[i-1]
        t3[i] = s3[i]+t3[i-1]
        t4[i] = s4[i]+t4[i-1]
      else:
        t1[i] = s1[i]
        t2[i] = s2[i]
        t3[i] = s3[i]
        t4[i] = s4[i]
# Computing ranks corresponding to 0.1 in all 3 CDFs
  for i1 in range(N):
    if t1[i1] > 0.9:
      break;
  for i2 in range(N):
    if t2[i2] > 0.9:
      break;
  for i3 in range(N):
    if t3[i3] > 0.9:
      break;
  for i4 in range(N):
    if t4[i4] > 0.9:
      break;
  plotpar = {'axes.labelsize': 24,
               'text.fontsize': 24,
               'legend.fontsize': 20,
               'xtick.labelsize': 22,
               'ytick.labelsize': 22,
               'text.usetex': False}
  rcParams.update(plotpar)
  close(1)
  figure(num=1, figsize=(16,12), facecolor='w', edgecolor='k')
# Plotting
# Replace by next line for BW figure
# l1,l2,l3,l4 = plot(ind,t1,'k-',ind,t2,'k--',ind,t3,'k-.',ind,t4,'k:',antialiased=True,lw=2)
  l1,l2,l3,l4 = plot(ind,t1,'b-',ind,t2,'g--',ind,t3,'r-.',ind,t4,'k-',antialiased=True,lw=2)
  xlabel('decreasing popularity ranks')
  xticks((0,2000000),('0', '2000000'))
  yaxislabel = "Cumulated use credit share (in a 2 million contributors universe)"
# Replace by next line for BW figure
# axhline(y=0.9, xmin=0, xmax=N, color='k',linestyle='dotted', lw=2)
  axhline(y=0.9, xmin=0, xmax=N, color='#666666',linestyle='dotted', lw=2)
  axvline(x=i1, ymin=0.0, ymax=t1[i1], color='k',linestyle='dotted', lw=2)
  annotate('%d' % (i1+1), (i1+10000,0.01), xytext=None, xycoords='data', textcoords='data', arrowprops=None, fontsize=20)
  axvline(x=i2, ymin=0.0, ymax=t2[i2], color='k',linestyle='dotted', lw=2)
  annotate('%d' % (i2+1), (i2+10000,0.01), xytext=None, xycoords='data', textcoords='data', arrowprops=None, fontsize=20)
  axvline(x=i3, ymin=0.0, ymax=t3[i3], color='k',linestyle='dotted', lw=2)
  annotate('%d' % (i3+1), (i3+10000,0.01), xytext=None, xycoords='data', textcoords='data', arrowprops=None, fontsize=20)
  axvline(x=i4, ymin=0.0, ymax=t4[i4], color='k',linestyle='dotted', lw=2)
  annotate('%d' % (i4+1), (i4+10000,0.01), xytext=None, xycoords='data', textcoords='data', arrowprops=None, fontsize=20)
  ylabel(yaxislabel)
  ylim(0.0, 1.0)
  legend( (l1,l2,l3,l4),  ('Existing copyright','Concentrated','Intermediate','Diverse'), loc='right')
  grid(False)
  savefig('illustration.png')
  show()

if __name__ == '__main__':
    main(sys.argv)
