# -*- coding: iso-8859-15 -*-
#    Loglog plotting of an observed popularity distribution
#
#    To generate figure A-3 of "Sharing", one has to run:
#    python figureA-3.py first1000-ML.csv
#    Copyright (C) 2009 - Philippe Aigrain
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA

from pylab import *
import csv

def harmonic(s,n):
  k=0.0;
  for i in arange (1.0,n,1.0):
    k += pow(i,-s);
  return k;  

def main(argv):
  if len(argv) < 2:
    print ("Usage: %s <CSV file containing data 2-uples>") % argv[0]
    sys.exit(0)
  # Let's first compute the number of lines in the data file
  cr = csv.reader(open(argv[1],"rb"))
  N = 0
  for row in cr:
    N += 1
  # Reading data
  data = reshape(zeros(N*2,float),(N,2))
  cr = csv.reader(open(argv[1],"rb"))
  i = 0
  for row in cr:
    data[i,0] = 1.0*(i+1)
    data[i,1] = 1.0*(int) (row[0])
    i += 1
  # Computing once and for all the observed cumulated distribution
  observed = zeros(N,float)
  for i in range(N-1):
    observed[i] = data[i+1,1]
  ind = range(1,N+1,1)
  plotpar = {'axes.labelsize': 24,
               'text.fontsize': 24,
               'legend.fontsize': 20,
               'xtick.labelsize': 22,
               'ytick.labelsize': 22,
               'text.usetex': False}
  rcParams.update(plotpar)
  close(1)
  figure(num=1, figsize=(16,12), facecolor='w', edgecolor='k')
# Plotting
# Replace by next line for BW figure
  loglog(ind,observed,'k-',lw=2)
  loglog(ind,observed,lw=2)
  ylim(0.0, 200000.0)
  xlabel('rank')
  ylabel('popularity')
  grid(True)
  savefig('illustration.png')
  show()

if __name__ == '__main__':
  main(sys.argv)
