# -*- coding: iso-8859-15 -*-
#    Plot of highest rewards depending on number of works, minimum reward,
#    Zipf's law paramter for various reward functions
#
#    To generate figure B-1 of "Sharing", one has to run
#    python figureB-1.py 1000000 40.0 0.9
#
#    Copyright (C) 2010 - Philippe Aigrain
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA

from pylab import *

def harmonic(s,n):
  k=0.0;
  for i in arange (1.0,n+1.0,1.0):
    k += pow(i,-s);
  return k;  

def main(argv):
  if len(argv) < 4:
    print ("Usage: %s <number of works> <min reward> <Zipf's law parameter>") % argv[0]
    print "Various options for reward functions"
    sys.exit(0)
  # Args
  N = float(argv[1])
  minreward = float(argv[2])
  a = float(argv[3])
  # Inits
  proportional = zeros(N,float)
  power067 = zeros(N,float)
  squareroot = zeros(N,float)
  cubicroot = zeros(N,float)
  totreward = minreward * harmonic(a,N) * pow(N,a)
  totrewardpower067 = minreward * harmonic(a*2.0/3.0,N) * pow(N,a*2.0/3.0)
  totrewardsquareroot = minreward * harmonic(a/2,N) * pow(N,a/2)
  totrewardcubicroot = minreward * harmonic(a/3,N) * pow(N,a/3)
  # computation of normalized rewards
  for i in arange(0.0,N-1.0,1.0):
    proportional[i] = minreward * pow(i+1.0,-a) * pow(N,a)
    power067[i] =  pow(i+1.0,-a*2.0/3.0) * pow(N,a*2.0/3.0) * minreward * totreward / totrewardsquareroot
    squareroot[i] =  pow(i+1.0,-a/2) * pow(N,a/2) * minreward * totreward / totrewardsquareroot
    cubicroot[i] =  pow(i+1.0,-a/3) * pow(N,a/3) * minreward * totreward / totrewardcubicroot
  ind = range(1,int(N)+1,1)
  plotpar = {'axes.labelsize': 24,
               'text.fontsize': 24,
               'legend.fontsize': 20,
               'xtick.labelsize': 22,
               'ytick.labelsize': 22,
               'text.usetex': False}
  rcParams.update(plotpar)
  close(1)
  figure(num=1, figsize=(16,12), facecolor='w', edgecolor='k')
# Plotting
  l1,l2,l3,l4= plot(ind,proportional,'k-',ind,power067,'k--',ind,squareroot,'k-.',ind,cubicroot,'k:',antialiased=True,lw=2)
# Replace by next line for a color figure
#  l1,l2,l3,l4= plot(ind,proportional,'b-',ind,power067,'g--',ind,squareroot,'r-.',ind,cubicroot,'k-',antialiased=True,lw=2)
  xlabel('decreasing popularity ranks')
  yaxislabel = 'reward'
  ylabel(yaxislabel)
  ylim(0.0, 200*minreward)
  xlim(1.0, N/50.0)
  legend( (l1,l2,l3,l4),  ('proportional','power(2/3)','square root','cube root'), loc='upper center')
  grid(True)
  savefig('illustration.png')
  show()



if __name__ == '__main__':
  main(sys.argv)
